import pandas as pd
import torch
import numpy as np
import subprocess
import os
import argparse

def run_mpnn(proteinmpnn_dir, pdb_file, device, output_prefix):
    cwd = os.getcwd() # Save current working directory
    os.chdir(proteinmpnn_dir) # Change directory to where MPNN scripts are centered at
    command = (
        'python ./score.py '
        '--model_type protein_mpnn '
        '--checkpoint_protein_mpnn ./model_params/v_48_020.pt '
        '--seed 42 '
        '--single_aa_score 1 '
        f'--out_folder {cwd}/{output_prefix}/ProteinMPNN '
        f'--pdb_path {cwd}/{pdb_file} '
        '--use_sequence 1 '
        '--batch_size 1 '
        '--number_of_batches 20 '
        f'--device {device}'
    )
    process = subprocess.Popen(command, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)
    out, err = process.communicate() # for debugging
    # print(out)
    # print('\n')
    # print(err)
    os.chdir(cwd) # Change directory back to original
    output = torch.load(f'{output_prefix}/ProteinMPNN/{pdb_file.split("/")[-1].split(".")[0]}.pt')
    logits = output['logits'].mean(axis=0)
    return logits

def standardize_vocab(nplogits, output_prefix, pdb_file):
    amino_acids = ['A', 'R', 'N', 'D', 'C', 'Q', 'E', 'G', 'H', 'I', 'L', 'K', 'M', 'F', 'P', 'S', 'T', 'W', 'Y', 'V']
    mpnn_alphabet = {"A": 0,"C": 1,"D": 2,"E": 3,"F": 4,"G": 5,"H": 6,"I": 7,"K": 8,"L": 9,
    "M": 10,"N": 11,"P": 12,"Q": 13,"R": 14,"S": 15,"T": 16,"V": 17,"W": 18,"Y": 19
    }
    ### Changing vocab by summing logits ###
    new_nplogits = []
    for aa in amino_acids:
        new_nplogits.append(nplogits[:,mpnn_alphabet[aa]])
    new_nplogits = np.array(new_nplogits).T
    np.save(f'{output_prefix}/{pdb_file.split("/")[-1].split(".")[0]}.logits.ProteinMPNN_v_48_020.npy', new_nplogits)

def main(args):
    proteinmpnn_dir = args.proteinmpnn_dir
    pdb_file = args.pdb_file
    output_prefix = args.output_prefix
    device = args.device

    # make directory {output_prefix}/ProteinMPNN/ if it does not exist
    os.makedirs(f'{output_prefix}/ProteinMPNN/', exist_ok=True)

    nplogits = run_mpnn(proteinmpnn_dir, pdb_file, device, output_prefix).astype(np.float128)
    
    standardize_vocab(nplogits, output_prefix, pdb_file)


if __name__ == "__main__":
    parser = argparse.ArgumentParser(description='Process amino acid and 3Di sequences using SaProt model.')
    parser.add_argument('--proteinmpnn_dir', type=str, help='Path to the proteinmpnn directory')
    parser.add_argument('--pdb_file', type=str, help='Path to the pdb file')
    parser.add_argument('--output_prefix', type=str, help='Optional prefix to add to the output filename')
    parser.add_argument('--device', type=str, help='Specify the Cuda device index', default = '0')

    args = parser.parse_args()

    main(args)



import argparse
import requests

def main(args):
    # Input sequence
    prefix = f'{args.dir}/{args.name}'
    sequence = f'{args.sequence}'

    # Making a POST request to the ESMFold API
    url = "https://api.esmatlas.com/foldSequence/v1/pdb/"
    headers = {'Content-Type': 'text/plain'}
    pdb_str = requests.post(url, headers=headers, data=sequence, verify=False)  # `verify=False` ignores SSL certificate validation

    # Check if the request was successful
    if pdb_str.status_code == 200:
        with open(f"{prefix}.pdb", "w") as out:
            out.write(pdb_str.text)
        print(f"Output saved to {prefix}.pdb")
    else:
        print("Failed to retrieve data:", pdb_str.status_code)


if __name__ == '__main__':
    # argparsing
    parser = argparse.ArgumentParser(description='Fold a protein sequence using ESMFold API.')
    parser.add_argument('--name', type=str, help='ID in FASTA file')
    parser.add_argument('--sequence', type=str, help='Amino acid sequence to fold')
    parser.add_argument('--dir', type=str, help='Directory to save the output file')
    args = parser.parse_args()
    
    main(args)


import argparse
import json
import os.path
import random
import sys

import numpy as np
import torch

from data_utils import (
    element_dict_rev,
    alphabet,
    restype_int_to_str,
    featurize,
    parse_PDB,
)
from model_utils import ProteinMPNN


def main(args) -> None:
    """
    Inference function
    """
    if args.seed:
        seed = args.seed
    else:
        seed = int(np.random.randint(0, high=99999, size=1, dtype=int)[0])
    torch.manual_seed(seed)
    random.seed(seed)
    np.random.seed(seed)
    device = torch.device("cuda:"+args.device if (torch.cuda.is_available()) else "cpu")
    folder_for_outputs = args.out_folder
    base_folder = folder_for_outputs
    if base_folder[-1] != "/":
        base_folder = base_folder + "/"
    if not os.path.exists(base_folder):
        os.makedirs(base_folder, exist_ok=True)
    if args.model_type == "protein_mpnn":
        checkpoint_path = args.checkpoint_protein_mpnn
    elif args.model_type == "ligand_mpnn":
        checkpoint_path = args.checkpoint_ligand_mpnn
    elif args.model_type == "per_residue_label_membrane_mpnn":
        checkpoint_path = args.checkpoint_per_residue_label_membrane_mpnn
    elif args.model_type == "global_label_membrane_mpnn":
        checkpoint_path = args.checkpoint_global_label_membrane_mpnn
    elif args.model_type == "soluble_mpnn":
        checkpoint_path = args.checkpoint_soluble_mpnn
    else:
        print("Choose one of the available models")
        sys.exit()
    checkpoint = torch.load(checkpoint_path, map_location=device)
    if args.model_type == "ligand_mpnn":
        atom_context_num = checkpoint["atom_context_num"]
        ligand_mpnn_use_side_chain_context = args.ligand_mpnn_use_side_chain_context
        k_neighbors = checkpoint["num_edges"]
    else:
        atom_context_num = 1
        ligand_mpnn_use_side_chain_context = 0
        k_neighbors = checkpoint["num_edges"]

    model = ProteinMPNN(
        node_features=128,
        edge_features=128,
        hidden_dim=128,
        num_encoder_layers=3,
        num_decoder_layers=3,
        k_neighbors=k_neighbors,
        device=device,
        atom_context_num=atom_context_num,
        model_type=args.model_type,
        ligand_mpnn_use_side_chain_context=ligand_mpnn_use_side_chain_context,
    )

    model.load_state_dict(checkpoint["model_state_dict"])
    model.to(device)
    model.eval()

    if args.pdb_path_multi:
        with open(args.pdb_path_multi, "r") as fh:
            pdb_paths = list(json.load(fh))
    else:
        pdb_paths = [args.pdb_path]

    if args.fixed_residues_multi:
        with open(args.fixed_residues_multi, "r") as fh:
            fixed_residues_multi = json.load(fh)
    else:
        fixed_residues = [item for item in args.fixed_residues.split()]
        fixed_residues_multi = {}
        for pdb in pdb_paths:
            fixed_residues_multi[pdb] = fixed_residues

    if args.redesigned_residues_multi:
        with open(args.redesigned_residues_multi, "r") as fh:
            redesigned_residues_multi = json.load(fh)
    else:
        redesigned_residues = [item for item in args.redesigned_residues.split()]
        redesigned_residues_multi = {}
        for pdb in pdb_paths:
            redesigned_residues_multi[pdb] = redesigned_residues

    # loop over PDB paths
    for pdb in pdb_paths:
        if args.verbose:
            print("Designing protein from this path:", pdb)
        fixed_residues = fixed_residues_multi[pdb]
        redesigned_residues = redesigned_residues_multi[pdb]
        protein_dict, backbone, other_atoms, icodes, _ = parse_PDB(
            pdb,
            device=device,
            chains=args.parse_these_chains_only,
            parse_all_atoms=args.ligand_mpnn_use_side_chain_context,
            parse_atoms_with_zero_occupancy=args.parse_atoms_with_zero_occupancy
        )
        # make chain_letter + residue_idx + insertion_code mapping to integers
        R_idx_list = list(protein_dict["R_idx"].cpu().numpy())  # residue indices
        chain_letters_list = list(protein_dict["chain_letters"])  # chain letters
        encoded_residues = []
        for i, R_idx_item in enumerate(R_idx_list):
            tmp = str(chain_letters_list[i]) + str(R_idx_item) + icodes[i]
            encoded_residues.append(tmp)
        encoded_residue_dict = dict(zip(encoded_residues, range(len(encoded_residues))))
        encoded_residue_dict_rev = dict(
            zip(list(range(len(encoded_residues))), encoded_residues)
        )

        fixed_positions = torch.tensor(
            [int(item not in fixed_residues) for item in encoded_residues],
            device=device,
        )
        redesigned_positions = torch.tensor(
            [int(item not in redesigned_residues) for item in encoded_residues],
            device=device,
        )

        # specify which residues are buried for checkpoint_per_residue_label_membrane_mpnn model
        if args.transmembrane_buried:
            buried_residues = [item for item in args.transmembrane_buried.split()]
            buried_positions = torch.tensor(
                [int(item in buried_residues) for item in encoded_residues],
                device=device,
            )
        else:
            buried_positions = torch.zeros_like(fixed_positions)

        if args.transmembrane_interface:
            interface_residues = [item for item in args.transmembrane_interface.split()]
            interface_positions = torch.tensor(
                [int(item in interface_residues) for item in encoded_residues],
                device=device,
            )
        else:
            interface_positions = torch.zeros_like(fixed_positions)
        protein_dict["membrane_per_residue_labels"] = 2 * buried_positions * (
            1 - interface_positions
        ) + 1 * interface_positions * (1 - buried_positions)

        if args.model_type == "global_label_membrane_mpnn":
            protein_dict["membrane_per_residue_labels"] = (
                args.global_transmembrane_label + 0 * fixed_positions
            )
        if type(args.chains_to_design) == str:
            chains_to_design_list = args.chains_to_design.split(",")
        else:
            chains_to_design_list = protein_dict["chain_letters"]
        chain_mask = torch.tensor(
            np.array(
                [
                    item in chains_to_design_list
                    for item in protein_dict["chain_letters"]
                ],
                dtype=np.int32,
            ),
            device=device,
        )

        # create chain_mask to notify which residues are fixed (0) and which need to be designed (1)
        if redesigned_residues:
            protein_dict["chain_mask"] = chain_mask * (1 - redesigned_positions)
        elif fixed_residues:
            protein_dict["chain_mask"] = chain_mask * fixed_positions
        else:
            protein_dict["chain_mask"] = chain_mask

        if args.verbose:
            PDB_residues_to_be_redesigned = [
                encoded_residue_dict_rev[item]
                for item in range(protein_dict["chain_mask"].shape[0])
                if protein_dict["chain_mask"][item] == 1
            ]
            PDB_residues_to_be_fixed = [
                encoded_residue_dict_rev[item]
                for item in range(protein_dict["chain_mask"].shape[0])
                if protein_dict["chain_mask"][item] == 0
            ]
            print("These residues will be redesigned: ", PDB_residues_to_be_redesigned)
            print("These residues will be fixed: ", PDB_residues_to_be_fixed)

        # specify which residues are linked
        if args.symmetry_residues:
            symmetry_residues_list_of_lists = [
                x.split(",") for x in args.symmetry_residues.split("|")
            ]
            remapped_symmetry_residues = []
            for t_list in symmetry_residues_list_of_lists:
                tmp_list = []
                for t in t_list:
                    tmp_list.append(encoded_residue_dict[t])
                remapped_symmetry_residues.append(tmp_list)
        else:
            remapped_symmetry_residues = [[]]

        if args.homo_oligomer:
            if args.verbose:
                print("Designing HOMO-OLIGOMER")
            chain_letters_set = list(set(chain_letters_list))
            reference_chain = chain_letters_set[0]
            lc = len(reference_chain)
            residue_indices = [
                item[lc:] for item in encoded_residues if item[:lc] == reference_chain
            ]
            remapped_symmetry_residues = []
            for res in residue_indices:
                tmp_list = []
                tmp_w_list = []
                for chain in chain_letters_set:
                    name = chain + res
                    tmp_list.append(encoded_residue_dict[name])
                    tmp_w_list.append(1 / len(chain_letters_set))
                remapped_symmetry_residues.append(tmp_list)

        # set other atom bfactors to 0.0
        if other_atoms:
            other_bfactors = other_atoms.getBetas()
            other_atoms.setBetas(other_bfactors * 0.0)

        # adjust input PDB name by dropping .pdb if it does exist
        name = pdb[pdb.rfind("/") + 1 :]
        if name[-4:] == ".pdb":
            name = name[:-4]

        with torch.no_grad():
            # run featurize to remap R_idx and add batch dimension
            if args.verbose:
                if "Y" in list(protein_dict):
                    atom_coords = protein_dict["Y"].cpu().numpy()
                    atom_types = list(protein_dict["Y_t"].cpu().numpy())
                    atom_mask = list(protein_dict["Y_m"].cpu().numpy())
                    number_of_atoms_parsed = np.sum(atom_mask)
                else:
                    print("No ligand atoms parsed")
                    number_of_atoms_parsed = 0
                    atom_types = ""
                    atom_coords = []
                if number_of_atoms_parsed == 0:
                    print("No ligand atoms parsed")
                elif args.model_type == "ligand_mpnn":
                    print(
                        f"The number of ligand atoms parsed is equal to: {number_of_atoms_parsed}"
                    )
                    for i, atom_type in enumerate(atom_types):
                        print(
                            f"Type: {element_dict_rev[atom_type]}, Coords {atom_coords[i]}, Mask {atom_mask[i]}"
                        )
            feature_dict = featurize(
                protein_dict,
                cutoff_for_score=args.ligand_mpnn_cutoff_for_score,
                use_atom_context=args.ligand_mpnn_use_atom_context,
                number_of_ligand_atoms=atom_context_num,
                model_type=args.model_type,
            )
            feature_dict["batch_size"] = args.batch_size
            B, L, _, _ = feature_dict["X"].shape  # batch size should be 1 for now.
            # add additional keys to the feature dictionary
            feature_dict["symmetry_residues"] = remapped_symmetry_residues

            logits_list = []
            probs_list = []
            log_probs_list = []
            decoding_order_list = []
            for _ in range(args.number_of_batches):
                feature_dict["randn"] = torch.randn(
                    [feature_dict["batch_size"], feature_dict["mask"].shape[1]],
                    device=device,
                )
                if args.autoregressive_score:
                    score_dict = model.score(feature_dict, use_sequence=args.use_sequence)
                elif args.single_aa_score:
                    score_dict = model.single_aa_score(feature_dict, use_sequence=args.use_sequence)
                else:
                    print("Set either autoregressive_score or single_aa_score to True")
                    sys.exit()
                logits_list.append(score_dict["logits"])
                log_probs_list.append(score_dict["log_probs"])
                probs_list.append(torch.exp(score_dict["log_probs"]))
                decoding_order_list.append(score_dict["decoding_order"])
            log_probs_stack = torch.cat(log_probs_list, 0)
            logits_stack = torch.cat(logits_list, 0)
            probs_stack = torch.cat(probs_list, 0)
            decoding_order_stack = torch.cat(decoding_order_list, 0)

            output_stats_path = base_folder + name + args.file_ending + ".pt"
            out_dict = {}
            out_dict["logits"] = logits_stack.cpu().numpy()
            out_dict["probs"] = probs_stack.cpu().numpy()
            out_dict["log_probs"] = log_probs_stack.cpu().numpy()
            out_dict["decoding_order"] = decoding_order_stack.cpu().numpy()
            out_dict["native_sequence"] = feature_dict["S"][0].cpu().numpy()
            out_dict["mask"] = feature_dict["mask"][0].cpu().numpy()
            out_dict["chain_mask"] = feature_dict["chain_mask"][0].cpu().numpy() #this affects decoding order
            out_dict["seed"] = seed
            out_dict["alphabet"] = alphabet
            out_dict["residue_names"] = encoded_residue_dict_rev

            mean_probs = np.mean(out_dict["probs"], 0)
            std_probs = np.std(out_dict["probs"], 0)
            sequence = [restype_int_to_str[AA] for AA in out_dict["native_sequence"]]
            mean_dict = {}
            std_dict = {}
            for residue in range(L):
                mean_dict_ = dict(zip(alphabet, mean_probs[residue]))
                mean_dict[encoded_residue_dict_rev[residue]] = mean_dict_
                std_dict_ = dict(zip(alphabet, std_probs[residue]))
                std_dict[encoded_residue_dict_rev[residue]] = std_dict_

            out_dict["sequence"] = sequence
            out_dict["mean_of_probs"] = mean_dict
            out_dict["std_of_probs"] = std_dict
            torch.save(out_dict, output_stats_path)



if __name__ == "__main__":
    argparser = argparse.ArgumentParser(
        formatter_class=argparse.ArgumentDefaultsHelpFormatter
    )

    argparser.add_argument(
        "--model_type",
        type=str,
        default="protein_mpnn",
        help="Choose your model: protein_mpnn, ligand_mpnn, per_residue_label_membrane_mpnn, global_label_membrane_mpnn, soluble_mpnn",
    )
    # protein_mpnn - original ProteinMPNN trained on the whole PDB exluding non-protein atoms
    # ligand_mpnn - atomic context aware model trained with small molecules, nucleotides, metals etc on the whole PDB
    # per_residue_label_membrane_mpnn - ProteinMPNN model trained with addition label per residue specifying if that residue is buried or exposed
    # global_label_membrane_mpnn - ProteinMPNN model trained with global label per PDB id to specify if protein is transmembrane
    # soluble_mpnn - ProteinMPNN trained only on soluble PDB ids
    argparser.add_argument(
        "--checkpoint_protein_mpnn",
        type=str,
        default="./model_params/proteinmpnn_v_48_020.pt",
        help="Path to model weights.",
    )
    argparser.add_argument(
        "--checkpoint_ligand_mpnn",
        type=str,
        default="./model_params/ligandmpnn_v_32_010_25.pt",
        help="Path to model weights.",
    )
    argparser.add_argument(
        "--checkpoint_per_residue_label_membrane_mpnn",
        type=str,
        default="./model_params/per_residue_label_membrane_mpnn_v_48_020.pt",
        help="Path to model weights.",
    )
    argparser.add_argument(
        "--checkpoint_global_label_membrane_mpnn",
        type=str,
        default="./model_params/global_label_membrane_mpnn_v_48_020.pt",
        help="Path to model weights.",
    )
    argparser.add_argument(
        "--checkpoint_soluble_mpnn",
        type=str,
        default="./model_params/solublempnn_v_48_020.pt",
        help="Path to model weights.",
    )

    argparser.add_argument("--verbose", type=int, default=1, help="Print stuff")

    argparser.add_argument(
        "--pdb_path", type=str, default="", help="Path to the input PDB."
    )
    argparser.add_argument(
        "--pdb_path_multi",
        type=str,
        default="",
        help="Path to json listing PDB paths. {'/path/to/pdb': ''} - only keys will be used.",
    )

    argparser.add_argument(
        "--fixed_residues",
        type=str,
        default="",
        help="Provide fixed residues, A12 A13 A14 B2 B25",
    )
    argparser.add_argument(
        "--fixed_residues_multi",
        type=str,
        default="",
        help="Path to json mapping of fixed residues for each pdb i.e., {'/path/to/pdb': 'A12 A13 A14 B2 B25'}",
    )

    argparser.add_argument(
        "--redesigned_residues",
        type=str,
        default="",
        help="Provide to be redesigned residues, everything else will be fixed, A12 A13 A14 B2 B25",
    )
    argparser.add_argument(
        "--redesigned_residues_multi",
        type=str,
        default="",
        help="Path to json mapping of redesigned residues for each pdb i.e., {'/path/to/pdb': 'A12 A13 A14 B2 B25'}",
    )

    argparser.add_argument(
        "--symmetry_residues",
        type=str,
        default="",
        help="Add list of lists for which residues need to be symmetric, e.g. 'A12,A13,A14|C2,C3|A5,B6'",
    )
    
    argparser.add_argument(
        "--homo_oligomer",
        type=int,
        default=0,
        help="Setting this to 1 will automatically set --symmetry_residues and --symmetry_weights to do homooligomer design with equal weighting.",
    )

    argparser.add_argument(
        "--out_folder",
        type=str,
        help="Path to a folder to output scores, e.g. /home/out/",
    )
    argparser.add_argument(
        "--file_ending", type=str, default="", help="adding_string_to_the_end"
    )
    argparser.add_argument(
        "--zero_indexed",
        type=str,
        default=0,
        help="1 - to start output PDB numbering with 0",
    )
    argparser.add_argument(
        "--seed",
        type=int,
        default=0,
        help="Set seed for torch, numpy, and python random.",
    )
    argparser.add_argument(
        "--batch_size",
        type=int,
        default=1,
        help="Number of sequence to generate per one pass.",
    )
    argparser.add_argument(
        "--number_of_batches",
        type=int,
        default=1,
        help="Number of times to design sequence using a chosen batch size.",
    )

    argparser.add_argument(
        "--ligand_mpnn_use_atom_context",
        type=int,
        default=1,
        help="1 - use atom context, 0 - do not use atom context.",
    )

    argparser.add_argument(
        "--ligand_mpnn_use_side_chain_context",
        type=int,
        default=0,
        help="Flag to use side chain atoms as ligand context for the fixed residues",
    )

    argparser.add_argument(
        "--ligand_mpnn_cutoff_for_score",
        type=float,
        default=8.0,
        help="Cutoff in angstroms between protein and context atoms to select residues for reporting score.",
    )

    argparser.add_argument(
        "--chains_to_design",
        type=str,
        default=None,
        help="Specify which chains to redesign, all others will be kept fixed.",
    )

    argparser.add_argument(
        "--parse_these_chains_only",
        type=str,
        default="",
        help="Provide chains letters for parsing backbones, 'ABCF'",
    )

    argparser.add_argument(
        "--transmembrane_buried",
        type=str,
        default="",
        help="Provide buried residues when using checkpoint_per_residue_label_membrane_mpnn model, A12 A13 A14 B2 B25",
    )
    argparser.add_argument(
        "--transmembrane_interface",
        type=str,
        default="",
        help="Provide interface residues when using checkpoint_per_residue_label_membrane_mpnn model, A12 A13 A14 B2 B25",
    )

    argparser.add_argument(
        "--global_transmembrane_label",
        type=int,
        default=0,
        help="Provide global label for global_label_membrane_mpnn model. 1 - transmembrane, 0 - soluble",
    )

    argparser.add_argument(
        "--parse_atoms_with_zero_occupancy",
        type=int,
        default=0,
        help="To parse atoms with zero occupancy in the PDB input files. 0 - do not parse, 1 - parse atoms with zero occupancy",
    )

    argparser.add_argument(
        "--use_sequence",
        type=int,
        default=1,
        help="1 - get scores using amino acid sequence info; 0 - get scores using backbone info only",
    )

    argparser.add_argument(
        "--autoregressive_score",
        type=int,
        default=0,
        help="1 - run autoregressive scoring function; p(AA_1|backbone); p(AA_2|backbone, AA_1) etc, 0 - False",
    )

    argparser.add_argument(
        "--single_aa_score",
        type=int,
        default=1,
        help="1 - run single amino acid scoring function; p(AA_i|backbone, AA_{all except ith one}), 0 - False",
    )

    argparser.add_argument(
        "--device",
        type=str,
        default='0',
        help="Cuda device to use (default: 0)",
    )

    args = argparser.parse_args()
    main(args)



import argparse
from Bio.Blast import NCBIWWW, NCBIXML
from io import StringIO
import pandas as pd
from Bio.Seq import Seq
from Bio.SeqRecord import SeqRecord
from Bio import SeqIO
import os
import numpy as np
import subprocess

def perform_blast_search(sequence):
    print('Performing BLAST search...')
    blast_result = NCBIWWW.qblast(program="blastp", database="nr", sequence=sequence, url_base='https://blast.ncbi.nlm.nih.gov/Blast.cgi', hitlist_size=100, format_type='XML', expect=0.001, short_query=False)
    return blast_result.read()

def parse_blast_xml(xml_blast):
    result_handle = StringIO(xml_blast)
    return next(NCBIXML.parse(result_handle))

def create_df(blast_record, sequence):
    df = pd.DataFrame([{'description': 'query_sequence', 'seq': sequence}] +
                        [{'description': alignment.hit_def, 'seq': hsp.sbjct} for alignment in blast_record.alignments for hsp in alignment.hsps if hsp.expect < 0.04])
    return df

def write_sequences(df):
    records = [SeqRecord(Seq(row['seq']), id=row['description'], description='') for index, row in df.iterrows()]
    # make dir if it does not exist
    SeqIO.write(records, f'{args.outdir}/BLAST/all_sequences.fasta', 'fasta')
    SeqIO.write(records[0], f'{args.outdir}/BLAST/query.fasta', 'fasta')
    SeqIO.write(records[1:], f'{args.outdir}/BLAST/targets.fasta', 'fasta')

def perform_msa():
    input_path = f'{args.outdir}/BLAST/all_sequences.fasta'
    output_path = f'{args.outdir}/BLAST/msa.fasta'
    
    # Build the command as a list
    command = ['mafft', '--auto', input_path]
    
    # Execute the command, capturing stdout (the MSA result)
    result = subprocess.run(command, capture_output=True, text=True, check=True)
    
    # Write the MSA result to the output file
    with open(output_path, 'w') as handle:
        handle.write(result.stdout)

    # Optionally, handle stderr or errors
    if result.stderr:
        print("MAFFT generated warnings/errors:", result.stderr)

def calculate_percentages():
    sequences = [str(record.seq) for record in SeqIO.parse(f'{args.outdir}/BLAST/msa.fasta', "fasta")]
    non_gap_indices = [i for i, char in enumerate(sequences[0]) if char != '-']
    filtered_sequences = [''.join(seq[i] for i in non_gap_indices) for seq in sequences]
    transposed_sequences = list(map(list, zip(*filtered_sequences)))

    # Create a DataFrame from the transposed list for easier manipulation.
    df = pd.DataFrame(transposed_sequences)

    aa_list = ['A', 'R', 'N', 'D', 'C', 'Q', 'E', 'G', 'H', 'I', 'L', 'K', 'M', 'F', 'P', 'S', 'T', 'W', 'Y', 'V']

    # Initialize an empty DataFrame to store frequencies
    percentage_df = pd.DataFrame(0, columns=aa_list, index=range(len(df)), dtype=float)

    # Calculate frequencies.
    for index, row in df.iterrows():
        row_counts = row.value_counts(normalize=True)  # Get normalized counts as frequencies directly.
        for aa in aa_list:
            if aa in row_counts:  # Check if the amino acid is present in the row_counts.
                percentage_df.at[index, aa] = row_counts.get(aa, 0)  # Update frequency.

    # softmax across columns
    percentage_df = percentage_df.div(percentage_df.sum(axis=1), axis=0)

    return percentage_df


def main(args):
    os.makedirs(args.outdir, exist_ok=True)
    os.makedirs(args.outdir + '/BLAST', exist_ok=True)

    xml_blast = perform_blast_search(args.sequence)
    blast_record = parse_blast_xml(xml_blast)
    df = create_df(blast_record, args.sequence)
    write_sequences(df)
    perform_msa()
    df = calculate_percentages()
    
    # save to np array
    np.save(f"{args.outdir}/{args.name}.logits.blast_phylogeny.npy", df.to_numpy())

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description='BLAST search and analysis')
    parser.add_argument('--sequence', type=str, required=True, help='Amino acid sequence for BLAST search')
    parser.add_argument('--outdir', type=str, required=True, help='Output directory for results')
    parser.add_argument('--name', type=str, required=True, help='Name of the sequence')
    args = parser.parse_args()

    main(args)
    


from transformers import AutoTokenizer, AutoModelForMaskedLM
import torch
import numpy as np
import argparse

print("ESM started running.")

def load_model(model_choice):
    print('Loading ESM2 model...')
    if model_choice == 'esm2_t48_15B_UR50D':
        tokenizer = AutoTokenizer.from_pretrained("facebook/esm2_t48_15B_UR50D")
        model = AutoModelForMaskedLM.from_pretrained("facebook/esm2_t48_15B_UR50D")
        return model, tokenizer
    elif model_choice == 'esm2_t36_3B_UR50D':
        tokenizer = AutoTokenizer.from_pretrained("facebook/esm2_t36_3B_UR50D")
        model = AutoModelForMaskedLM.from_pretrained("facebook/esm2_t36_3B_UR50D")
        return model, tokenizer
    elif model_choice == 'esm2_t33_650M_UR50D':
        tokenizer = AutoTokenizer.from_pretrained("facebook/esm2_t33_650M_UR50D")
        model = AutoModelForMaskedLM.from_pretrained("facebook/esm2_t33_650M_UR50D")
        return model, tokenizer
    else:
        print(f"Invalid model choice: {model_choice}. Running with default model esm2_t33_650M_UR50D")
        tokenizer = AutoTokenizer.from_pretrained("facebook/esm2_t33_650M_UR50D")
        model = AutoModelForMaskedLM.from_pretrained("facebook/esm2_t33_650M_UR50D")
        return model, tokenizer

def tokenize_sequences(data, tokenizer, device):
    print('Tokenizing sequences...')
    tokens = tokenizer(data, return_tensors="pt")
    tokens = {k: v.to(device) for k, v in tokens.items()}
    return tokens

def process_sequences(model, tokens):
    print('Processing sequences...')
    with torch.no_grad():
        outputs = model(**tokens)
    logits = outputs['logits'][0]
    return logits.cpu().numpy().astype(np.float128)

def standardize_vocab(tokenizer,nplogits,model_choice,output_prefix):
    amino_acids = ['A', 'R', 'N', 'D', 'C', 'Q', 'E', 'G', 'H', 'I', 'L', 'K', 'M', 'F', 'P', 'S', 'T', 'W', 'Y', 'V']
    ### Changing vocab by summing logits ###
    vocab = tokenizer.get_vocab()
    filtered_vocab = {token: index for token, index in vocab.items() if token in amino_acids} # Saving only amino acid {token:index} pairs

    new_nplogits = []
    for aa in amino_acids:
        new_nplogits.append(nplogits[1:-1,filtered_vocab[aa]])
    new_nplogits = np.array(new_nplogits).T
    np.save(f'{output_prefix}.logits.{model_choice}.npy', new_nplogits)

def main(args):

    sequence = args.sequence
    model_choice = args.model_choice
    output_prefix = args.output_prefix
    device = args.device

    model, tokenizer = load_model(model_choice)
    device = torch.device(f"cuda:{device}" if torch.cuda.is_available() else "cpu")
    model = model.to(device)
    tokens = tokenize_sequences(sequence, tokenizer, device)

    nplogits = process_sequences(model, tokens)
    standardize_vocab(tokenizer,nplogits,model_choice,output_prefix)

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description='Process sequences using ESM-2 model.')
    parser.add_argument('--sequence', type=str, help='Sequence to pass through ESM-2 model')
    parser.add_argument('--model_choice', type=str, help='Choice of ESM2 model. Options: [esm2_t48_15B_UR50D, esm2_t36_3B_UR50D, esm2_t33_650M_UR50D] Default: esm2_t33_650M_UR50D', default = 'esm2_t33_650M_UR50D')
    parser.add_argument('--output_prefix', type=str, help='Optional prefix to add to the output filename')
    parser.add_argument('--device', type=str, help='Specify the Cuda device index', default = '0')

    args = parser.parse_args()
    main(args)



import argparse
import numpy as np
import torch
from transformers import AutoTokenizer, AutoModelForMaskedLM


def get_data(fasta_file):
    data = []
    with open(fasta_file, 'r') as inf:
        header, seq = None, None
        for line in inf:
            if line.startswith('>'):
                if header and seq:
                    data.append((header, seq))
                header, seq = line.strip('\n')[1:], ''
            else:
                seq += line.strip()
        if header and seq:
            data.append((header, seq))
    return data

def combine_data(amino_acids_str, foldseek_str):
    foldseek_str = foldseek_str.lower()
    sequence = ''
    for i in range(len(foldseek_str)):
        sequence += amino_acids_str[i]+foldseek_str[i]
    return sequence

def load_model():
    print('Loading SaProt model...')
    tokenizer = AutoTokenizer.from_pretrained("westlake-repl/SaProt_650M_PDB")
    model = AutoModelForMaskedLM.from_pretrained("westlake-repl/SaProt_650M_PDB")
    return model, tokenizer

def tokenize_sequences(data, tokenizer,device):
    print('Tokenizing sequences...')
    tokens = tokenizer(data, return_tensors="pt")
    tokens = {k: v.to(device) for k, v in tokens.items()}
    return tokens

def process_sequences(model, tokens):
    print('Processing sequences...')
    with torch.no_grad():
        outputs = model(**tokens)
    logits = outputs['logits'][0]
    return logits.cpu().numpy().astype(np.float128)

def standardize_vocab(tokenizer,nplogits,output_prefix):
    amino_acids = ['A', 'R', 'N', 'D', 'C', 'Q', 'E', 'G', 'H', 'I', 'L', 'K', 'M', 'F', 'P', 'S', 'T', 'W', 'Y', 'V']
    ### Changing vocab by summing logits ###
    vocab = tokenizer.get_vocab()
    filtered_vocab = {token: index for token, index in vocab.items() if "<" not in token and "#" not in token} # remove special tokens

    new_nplogits = []
    for aa in amino_acids:
        aa_ls = []
        for key,item in filtered_vocab.items():
            if aa in key:
                aa_ls.append(item)
        new_nplogits.append(np.sum(nplogits[1:-1,aa_ls],axis=1))
    new_nplogits = np.array(new_nplogits).T
    
    np.save(f'{output_prefix}.logits.SaProt_650M_PDB.npy', new_nplogits)


def main(args):
    amino_acid_str = args.amino_acid_str
    foldseek_tokens_input = args.foldseek_tokens
    output_prefix = args.output_prefix
    device = args.device

    model, tokenizer = load_model()
    device = torch.device("cuda:"+device if torch.cuda.is_available() else "cpu")
    model = model.to(device)
    foldseek_str = get_data(foldseek_tokens_input)[0][1]
    data = combine_data(amino_acid_str, foldseek_str)
    batch_tokens = tokenize_sequences(data, tokenizer,device)

    nplogits = process_sequences(model, batch_tokens)
    standardize_vocab(tokenizer,nplogits,output_prefix)

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description='Process amino acid and 3Di sequences using SaProt model.')
    parser.add_argument('--amino_acid_str', type=str, help='Amino acid sequence')
    parser.add_argument('--foldseek_tokens', type=str, help='Path to the amino acid fasta file')
    parser.add_argument('--output_prefix', type=str, help='Optional prefix to add to the output filename',default = '')
    parser.add_argument('--device', type=str, help='Specify the Cuda device index',default = '0')

    args = parser.parse_args()

    main(args)





def main(args):
    import os
    import sys
    import subprocess
    
    script_dir = os.path.dirname(os.path.realpath(__file__))

    os.makedirs(args.output_dir, exist_ok=True)

    if not os.path.isfile(args.fasta_file):
        print(f"Error: The file {args.fasta_file} does not exist.")
        sys.exit(1)

    with open(args.fasta_file) as f:
        for line in f:
            if line.startswith('>'):
                name = line[1:].strip()
                break

    with open(args.fasta_file) as f:
        seq = ''.join(line.strip() for line in f if not line.startswith('>'))

    os.makedirs(args.output_dir, exist_ok=True)

    print(f"Running ESMFold for {name}")
    subprocess.run(f"python3 {script_dir}/esm_fold.py --sequence {seq} --name {name} --dir {args.output_dir}", shell=True)

    pdb_file = os.path.join(args.output_dir, f"{name}.pdb")

    ############### Running Models ##########################################################################################
    print(f"Running models for {name}")

    if args.saprot:
        os.makedirs(f"{args.output_dir}/foldseek", exist_ok=True)
        subprocess.run(f"foldseek createdb {pdb_file} {args.output_dir}/foldseek/{name} -v 1", shell=True)
        subprocess.run(f"foldseek lndb {args.output_dir}/foldseek/{name}_h {args.output_dir}/foldseek/{name}_ss_h -v 1", shell=True)
        subprocess.run(f"foldseek convert2fasta {args.output_dir}/foldseek/{name}_ss {args.output_dir}/foldseek/{name}_ss.fasta -v 1", shell=True)
        print("Running SaProt...")
        subprocess.run(f"python3 {script_dir}/SaProt_getLogits.py --amino_acid_str {seq} --foldseek_tokens {args.output_dir}/foldseek/{name}_ss.fasta --output_prefix {args.output_dir}/{name}", shell=True)

    if args.antiberta:
        print("Running AntiBERTa2...")
        subprocess.run(f"python3 {script_dir}/AntiBERTa2_getLogits.py --amino_acid_str {seq} --output_prefix {args.output_dir}/{name}", shell=True)

    if args.esm:
        print("Running ESM2 (650M)...")
        subprocess.run(f"python3 {script_dir}/ESM2_getLogits.py --sequence {seq} --model_choice esm2_t33_650M_UR50D --output_prefix {args.output_dir}/{name}", shell=True)

    if args.proteinmpnn:
        proteinmpnn_dir = os.path.join(args.output_dir, "../LigandMPNN")
        if not os.path.isdir(proteinmpnn_dir):
            print("Setting up ProteinMPNN...")
            subprocess.run(f"git clone https://github.com/dauparas/LigandMPNN.git {proteinmpnn_dir}", shell=True)
            os.makedirs(f"{proteinmpnn_dir}/model_params", exist_ok=True)
            subprocess.run(f"wget https://github.com/dauparas/ProteinMPNN/raw/main/vanilla_model_weights/v_48_020.pt -P {proteinmpnn_dir}/model_params", shell=True)
            subprocess.run(f"cp {script_dir}/score.py {proteinmpnn_dir}/score.py", shell=True)
        else:
            print("ProteinMPNN already set up.")

        print("Running ProteinMPNN...")
        subprocess.run(f"python3 {script_dir}/ProteinMPNN_getLogits.py --output_prefix {args.output_dir} --proteinmpnn_dir {proteinmpnn_dir} --pdb_file {pdb_file}", shell=True)

    if args.blast:
        print("Running BLAST phylogeny...")
        subprocess.run(f"python3 {script_dir}/BLAST_phylo.py --outdir {args.output_dir} --sequence {seq} --name {name}", shell=True)


    ### MUTCOMPUTE SECITON ###
    if args.mutcompute:
        print("Running MutCompute...")
        docker_command = (
        f"docker run --rm -v $(pwd)/{args.output_dir}:/mutcompute/input "
        f"aaronfeller/penguin "
        f"-c 'cd scripts && python run.py -p {name}.pdb -d ../input/ > ../input/{name}.mutcompute.txt'"
        )
        
        # Execute the Docker command
        subprocess.run(docker_command, shell=True)

        print("Editing MutCompute output...")
        subprocess.run(f"python {script_dir}/edit_mutcompute_output.py --output {args.output_dir} --name {name}", shell=True)

    print(f"Predictions complete for {name}")

if __name__ == "__main__":
    import argparse
    parser = argparse.ArgumentParser(description="Process a FASTA file and run a sequence analysis pipeline.")
    parser.add_argument("fasta_file", type=str, help="Path to the input FASTA file.")
    parser.add_argument("output_dir", type=str, help="Path to the output directory.")
    parser.add_argument("--device", type=str, default="0", help="CUDA device number (default: 0)")
    parser.add_argument("--saprot", type=bool, default="True", help="Run SaProt")
    parser.add_argument("--esm", type=bool, default="True", help="Run ESM2")
    parser.add_argument("--proteinmpnn", type=bool, default="True", help="Run ProteinMPNN")
    parser.add_argument("--blast", type=bool, default="True", help="Run BLAST phylogeny")
    parser.add_argument("--mutcompute", type=bool, default="True", help="Run MutCompute")
    parser.add_argument("--antiberta", type=bool, default="False", help="Run AntiBERTa2")    
    args = parser.parse_args()
    main(args)



from setuptools import setup, find_packages
import os

# Directory containing this file
this_directory = os.path.abspath(os.path.dirname(__file__))
# Text of the README file
with open(os.path.join(this_directory, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='penguindashboard',
    version='1.0.11',
    author=['Aaron Feller', 'Phillip Woolley'],
    author_email=['aaronleefeller@gmail.com', 'prwoolley@utexas.edu'],
    description='PEngUIN: Protein Engineering Using Independent Networks',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/AaronFeller/PEngUIN',
    license='MIT',
    packages=find_packages(),
    include_package_data=True,
    entry_points={
        'console_scripts': [
            'penguin=penguindashboard.pipeline:main',
        ],
    },
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Science/Research',
        'Topic :: Scientific/Engineering :: Bio-Informatics',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
    ],
    python_requires='>=3.7',
)



import pandas as pd
import numpy as np
import argparse

def main(args):
    first_line = 'file,position,from,to,logodds,predH,predE,predD,predR,predK,predS,predT,predN,predQ,predA,predV,predL,predI,predM,predF,predY,predW,predP,predG,predC'
    
    # make first_line header of dataframe
    df = pd.DataFrame(columns=first_line.split(','))

    # read text file ../test_mc2/HUW04_protease.mutcompute.txt
    filename = f"{args.output}/{args.name}.mutcompute.txt"
    with open(filename) as f:
        lines = f.readlines()

    for line in lines:
        # if line starts with HUW04
        if line.startswith(args.name):
            # split line by comma
            data = line.split(',')
            # append line to dataframe
            df.loc[len(df)] = data
            
    # remove pred from column names
    df.columns = [col.replace('pred', '') for col in df.columns]

    # remove first 5 items of first_line
    df = df.drop(first_line.split(',')[:5], axis=1)
    amino_acids = ['A', 'R', 'N', 'D', 'C', 'Q', 'E', 'G', 'H', 'I', 'L', 'K', 'M', 'F', 'P', 'S', 'T', 'W', 'Y', 'V']

    # reorder df by amino acids
    df = df[amino_acids]
    # save to npy file
    df = df.astype(float)
    
    np.save(f'{args.output}/{args.name}.Mutcompute.npy', df.to_numpy())
    
if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument("--output", type=str, help="Path to the output directory.")
    parser.add_argument("--name", type=str, help="Name of the protein.")
    args = parser.parse_args()
    
    main(args)
